@@ -20,12 +20,13 @@ L{Artist}, L{Release}, and L{Track}.
 """
 from sets import Set
 
-__revision__ = '$Id: model.py 8728 2006-12-17 23:42:30Z luks $'
+__revision__ = '$Id: model.py 9724 2008-03-09 08:34:11Z matt $'
 
 __all__ = [
 	'VARIOUS_ARTISTS_ID', 'NS_MMD_1', 'NS_REL_1', 'NS_EXT_1', 
 	'Entity', 'Artist', 'Release', 'Track', 'User', 
-	'Relation', 'ArtistAlias', 'Disc', 'ReleaseEvent', 
+	'Relation', 'Disc', 'ReleaseEvent', 'Label', 'Tag',
+	'AbstractAlias', 'ArtistAlias', 'LabelAlias', 
 ]
 
 
@@ -42,8 +43,9 @@ class Entity(object):
 	"""A first-level MusicBrainz class.
 
 	All entities in MusicBrainz have unique IDs (which are absolute URIs)
-	and may have any number of L{relations <Relation>} to other entities.
-	This class is abstract and should not be instantiated.
+	as well as any number of L{relations <Relation>} to other entities
+	and free text tags. This class is abstract and should not be
+	instantiated.
 
 	Relations are differentiated by their I{target type}, that means,
 	where they link to. MusicBrainz currently supports four target types
@@ -69,6 +71,7 @@ class Entity(object):
 		"""
 		self._id = id_
 		self._relations = { }
+		self._tags = { }
 
 	def getId(self):
 		"""Returns a MusicBrainz ID.
@@ -109,7 +112,7 @@ class Entity(object):
 
 		If C{direction} is set, only relations with the given reading
 		direction are returned. You can use the L{Relation.DIR_FORWARD},
-		L{Relation.DIR_BACKWARD}, and L{Relation.DIR_BOTH} constants
+		L{Relation.DIR_BACKWARD}, and L{Relation.DIR_NONE} constants
 		for this.
 
 		@param targetType: a string containing an absolute URI, or None
@@ -217,6 +220,37 @@ class Entity(object):
 		"""
 		return self._relations.keys()
 
+	def getTag(self, value):
+		"""Return the tag with the given value (aka the tag's name).
+
+		@return: the L{Tag} with the given name or raises a KeyError
+		"""
+		return self._tags[value]
+
+	def getTags(self):
+		"""Return all tags attached to this Entity.
+
+		@return: a list of L{Tag} objects
+		"""
+		return self._tags.values()
+
+	tags = property(getTags, doc='The tags for this entity.')
+
+	def addTag(self, tag):
+		"""Add a new tag.
+
+		This merges an existing tag with the same name.
+
+		@param tag: the L{Tag} object to add
+
+		@see: L{getTags}
+		"""
+		if self._tags.has_key(tag.value):
+			existing = self._tags[tag.value]
+			existing.count += tag.count
+		else:
+			self._tags[tag.value] = tag
+
 
 class Artist(Entity):
 	"""Represents an artist.
@@ -383,7 +417,7 @@ class Artist(Entity):
 		doc="The begin/foundation date.")
 
 	def getEndDate(self):
-		"""Get the death/dissolving date.
+		"""Returns the death/dissolving date.
 
 		The definition of the I{end date} depends on the artist's
 		type. For persons, this is the day of death, for groups it
@@ -414,7 +448,7 @@ class Artist(Entity):
 		"""
 		return self._aliases
 
-	aliases = property(getAliases, doc='The list of aliases')
+	aliases = property(getAliases, doc='The list of aliases.')
 
 	def addAlias(self, alias):
 		"""Adds an alias for this artist.
@@ -495,6 +529,289 @@ class Artist(Entity):
 		doc='The total number of releases')
 
 
+class Tag(object):
+	"""The representation of a MusicBrainz folksonomy tag.
+
+	The tag's value is the text that's displayed in the tag cloud.
+	The count attribute keeps track of how many users added the tag
+	to its owning entity.
+	"""
+	def __init__(self, value=None, count=None):
+		"""Constructor.
+
+		@param value: a string containing the tag's value
+		@param count: the number of users who added this tag
+		"""
+		self._value = value
+		self._count = count
+
+	def getValue(self):
+		"""Returns a string with the tag's value.
+
+		@return: a string containing the tags's value, or None
+		"""
+		return self._value
+	
+	def setValue(self, value):
+		"""Sets the value of this tag.
+		
+		@param value: A string containing the value of the tag
+		"""
+		self._value = value
+		
+	value = property(getValue, setValue, doc='The value of the text.')
+
+	def getCount(self):
+		"""Returns an integer containing the tag's frequency count.
+
+		@return: an integer containing the tags's frequency count, or None
+		"""
+		return self._count
+	
+	def setCount(self, count):
+		"""Sets the frequency count of this tag.
+		
+		@param count: an integer containing the tag's frequency count
+		"""
+		self._count = count
+		
+	count = property(getCount, setCount, doc="This tag's frequency count.")
+
+	def __str__(self):
+		return str(self._value)
+
+	def __unicode__(self):
+		return unicode(self._value)
+
+
+class Label(Entity):
+	"""Represents a record label.
+	
+	A label within MusicBrainz is an L{Entity}. It contains information
+	about the label like when it was established, its name, label code and
+	other relationships. All release events may be assigned a label.
+	"""
+	TYPE_UNKNOWN = NS_MMD_1 + 'Unknown'
+	
+	TYPE_DISTRIBUTOR = NS_MMD_1 + 'Distributor'
+	TYPE_HOLDING = NS_MMD_1 + 'Holding'
+	TYPE_PRODUCTION = NS_MMD_1 + 'Production'
+	
+	TYPE_ORIGINAL = NS_MMD_1 + 'OriginalProduction'
+	TYPE_BOOTLEG = NS_MMD_1 + 'BootlegProduction'
+	TYPE_REISSUE = NS_MMD_1 + 'ReissueProduction'
+	
+	def __init__(self, id_=None):
+		"""Constructor.
+
+		@param id_: a string containing an absolute URI
+		"""
+		Entity.__init__(self, id_)
+		self._type = None
+		self._name = None
+		self._sortName = None
+		self._disambiguation = None
+		self._countryId = None
+		self._code = None
+		self._beginDate = None
+		self._endDate = None
+		self._aliases = [ ]
+	
+	def getType(self):
+		"""Returns the type of this label.
+
+		@return: a string containing an absolute URI
+		"""
+		return self._type
+	
+	def setType(self, type_):
+		"""Sets the type of this label.
+		
+		@param type_: A string containing the absolute URI of the type of label.
+		"""
+		self._type = type_
+	
+	type = property(getType, setType, doc='The type of label')
+	
+	def getName(self):
+		"""Returns a string with the name of the label.
+
+		@return: a string containing the label's name, or None
+		"""
+		return self._name
+	
+	def setName(self, name):
+		"""Sets the name of this label.
+		
+		@param name: A string containing the name of the label
+		"""
+		self._name = name
+		
+	name = property(getName, setName, doc='The name of the label.')
+	
+	def getSortName(self):
+		"""Returns the label's sort name.
+
+		The sort name is the label's name in a special format which
+		is better suited for lexicographic sorting. The MusicBrainz
+		style guide specifies this format.
+
+		@see: U{The MusicBrainz Style Guidelines
+			<http://musicbrainz.org/style.html>}
+		"""
+		return self._sortName
+
+	def setSortName(self, sortName):
+		"""Sets the label's sort name.
+
+		@param sortName: a string containing the label's sort name
+
+		@see: L{getSortName}
+		"""
+		self._sortName = sortName
+
+	sortName = property(getSortName, setSortName,
+		doc="The label's sort name.")
+
+	def getDisambiguation(self):
+		"""Returns the disambiguation attribute.
+
+		This attribute may be used if there is more than one label
+		with the same name. In this case, disambiguation attributes
+		are added to the labels' names to keep them apart.
+
+		@return: a disambiguation string, or None
+
+		@see: L{getUniqueName}
+		"""
+		return self._disambiguation
+
+	def setDisambiguation(self, disambiguation):
+		"""Sets the disambiguation attribute.
+
+		@param disambiguation: a disambiguation string
+
+		@see: L{getDisambiguation}, L{getUniqueName}
+		"""
+		self._disambiguation = disambiguation
+
+	disambiguation = property(getDisambiguation, setDisambiguation,
+		doc="The disambiguation comment.")
+
+	def getUniqueName(self):
+		"""Returns a unique label name (using disambiguation).
+
+		This method returns the label's name together with the
+		disambiguation attribute in parenthesis if it exists.
+
+		@return: a string containing the unique name
+
+		@see: L{getDisambiguation}
+		"""
+		d = self.getDisambiguation() 
+		if d is not None and d.strip() != '':
+			return '%s (%s)' % (self.getName(), d)
+		else: 
+			return self.getName()
+
+	def getBeginDate(self):
+		"""Returns the date this label was established.
+		
+		@return: A string contained the start date, or None
+		"""
+		return self._beginDate
+	
+	def setBeginDate(self, date):
+		"""Set the date this label was established.
+		
+		@param date: A string in the format of YYYY-MM-DD
+		"""
+		self._beginDate = date
+	
+	beginDate = property(getBeginDate, setBeginDate,
+		doc='The date this label was established.')
+
+	def getEndDate(self):
+		"""Returns the date this label closed.
+
+		The returned date has the format 'YYYY', 'YYYY-MM', or 
+		'YYYY-MM-DD', depending on how much detail is known.
+		
+		@return: A string containing the date, or None
+		"""
+		return self._endDate
+	
+	def setEndDate(self, date):
+		"""Set the date this label closed.
+
+		The date may have the format 'YYYY', 'YYYY-MM', or 
+		'YYYY-MM-DD', depending on how much detail is known.
+		
+		@param date: A string containing the date, or None
+		"""
+		self._endDate = date
+	
+	endDate = property(getEndDate, setEndDate,
+		doc='The date this label closed.')
+		
+	def getCountry(self):
+		"""Returns the country the label is located.
+
+		@return: a string containing an ISO-3166 country code, or None
+
+		@see: L{musicbrainz2.utils.getCountryName}
+		"""
+		return self._countryId
+
+	def setCountry(self, country):
+		"""Sets the country the label is located.
+
+		@param country: a string containing an ISO-3166 country code
+		"""
+		self._countryId = country
+
+	country = property(getCountry, setCountry,
+		doc='The country the label is located.')
+
+	def getCode(self):
+		"""Returns the label code.
+
+		Label codes have been introduced by the IFPI (International
+		Federation of Phonogram and Videogram Industries) to uniquely
+		identify record labels. The label code consists of 'LC-' and 4
+		figures (currently being extended to 5 figures).
+
+		@return: a string containing the label code, or None
+		"""
+		return self._code
+
+	def setCode(self, code):
+		"""Sets the label code.
+
+		@param code: a string containing the label code
+		"""
+		self._code = code
+
+	code = property(getCode, setCode,
+		doc='The label code.')
+
+	def getAliases(self):
+		"""Returns the list of aliases for this label.
+
+		@return: a list of L{LabelAlias} objects
+		"""
+		return self._aliases
+
+	aliases = property(getAliases, doc='The list of aliases.')
+
+	def addAlias(self, alias):
+		"""Adds an alias for this label.
+		
+		@param alias: a L{LabelAlias} object
+		"""
+		self._aliases.append(alias)
+
+
 class Release(Entity):
 	"""Represents a Release.
 
@@ -1055,9 +1372,10 @@ class Relation(object):
 	@cvar TO_TRACK: Identifies relations linking to a track.
 	@cvar TO_URL: Identifies relations linking to an URL.
 
-	@cvar DIR_BOTH: Relation reading direction doesn't matter.
+	@cvar DIR_NONE: Relation reading direction doesn't matter.
 	@cvar DIR_FORWARD: Relation reading direction is from source to target.
 	@cvar DIR_BACKWARD: Relation reading direction is from target to source.
+	@cvar DIR_BOTH: Relation reading direction doesn't matter (no longer used!).
 	"""
 	# Relation target types
 	#
@@ -1071,9 +1389,10 @@ class Relation(object):
 	DIR_BOTH = 'both'
 	DIR_FORWARD = 'forward'
 	DIR_BACKWARD = 'backward'
+	DIR_NONE = 'none'
 
 	def __init__(self, relationType=None, targetType=None, targetId=None,
-			direction=DIR_BOTH, attributes=None,
+			direction=DIR_NONE, attributes=None,
 			beginDate=None, endDate=None, target=None):
 		"""Constructor.
 
@@ -1081,7 +1400,7 @@ class Relation(object):
 		@param targetType: a string containing an absolute URI
 		@param targetId: a string containing an absolute URI
 		@param direction: one of C{Relation.DIR_FORWARD},
-		C{Relation.DIR_BACKWARD}, or C{Relation.DIR_BOTH}
+		C{Relation.DIR_BACKWARD}, or C{Relation.DIR_NONE}
 		@param attributes: a list of strings containing absolute URIs
 		@param beginDate: a string containing a date
 		@param endDate: a string containing a date
@@ -1230,15 +1549,16 @@ class Relation(object):
 		"""Returns the reading direction.
 
 		The direction may be one of L{Relation.DIR_FORWARD},
-		L{Relation.DIR_BACKWARD}, or L{Relation.DIR_BOTH},
+		L{Relation.DIR_BACKWARD}, or L{Relation.DIR_NONE},
 		depending on how the relation should be read. For example,
 		if direction is L{Relation.DIR_FORWARD} for a cover relation,
-		it is read as "X is a cover of Y". Some relations are
-		bidirectional, like marriages. In these cases, the direction
-		is L{Relation.DIR_BOTH}.
+		it is read as "X is a cover of Y". For some relations there is
+		no reading direction (like marriages) and the web service doesn't
+		send a direction. In these cases, the direction is set to
+		L{Relation.DIR_NONE}.
 
 		@return: L{Relation.DIR_FORWARD}, L{Relation.DIR_BACKWARD},
-		or L{Relation.DIR_BOTH}
+		or L{Relation.DIR_NONE}
 		"""
 		return self._direction
 
@@ -1246,7 +1566,7 @@ class Relation(object):
 		"""Sets the reading direction.
 
 		@param direction: L{Relation.DIR_FORWARD},
-		L{Relation.DIR_BACKWARD}, or L{Relation.DIR_BOTH}
+		L{Relation.DIR_BACKWARD}, or L{Relation.DIR_NONE}
 
 		@see: L{getDirection}
 		"""
@@ -1285,7 +1605,25 @@ class ReleaseEvent(object):
 	All country codes used must be valid ISO-3166 country codes (i.e. 'DE',
 	'UK' or 'FR'). The dates are strings and must have the format 'YYYY',
 	'YYYY-MM' or 'YYYY-MM-DD'.
+
+	The format of the release medium is a URI that can be compared to the
+	constants on this class (L{FORMAT_CD}, L{FORMAT_DVD} and others).
 	"""
+	FORMAT_CD = NS_MMD_1 + 'CD'
+	FORMAT_DVD = NS_MMD_1 + 'DVD'
+	FORMAT_SACD = NS_MMD_1 + 'SACD'
+	FORMAT_DUALDISC = NS_MMD_1 + 'DualDisc'
+	FORMAT_LASERDISC = NS_MMD_1 + 'LaserDisc'
+	FORMAT_MINIDISC = NS_MMD_1 + 'MiniDisc'
+	FORMAT_VINYL = NS_MMD_1 + 'Vinyl'
+	FORMAT_CASSETTE = NS_MMD_1 + 'Cassette'
+	FORMAT_CARTRIDGE = NS_MMD_1 + 'Cartridge'
+	FORMAT_REEL_TO_REEL = NS_MMD_1 + 'ReelToReel'
+	FORMAT_DAT = NS_MMD_1 + 'DAT'
+	FORMAT_DIGITAL = NS_MMD_1 + 'Digital'
+	FORMAT_WAX_CYLINDER = NS_MMD_1 + 'WaxCylinder'
+	FORMAT_PIANO_ROLL = NS_MMD_1 + 'PianoRoll'
+	FORMAT_OTHER = NS_MMD_1 + 'Other'
 
 	def __init__(self, country=None, dateStr=None):
 		"""Constructor.
@@ -1295,6 +1633,10 @@ class ReleaseEvent(object):
 		"""
 		self._countryId = country
 		self._dateStr = dateStr
+		self._catalogNumber = None
+		self._barcode = None
+		self._label = None
+		self._format = None
 
 	def getCountry(self):
 		"""Returns the country a release took place.
@@ -1319,6 +1661,56 @@ class ReleaseEvent(object):
 	country = property(getCountry, setCountry,
 		doc='The country a release took place.')
 
+	def getCatalogNumber(self):
+		"""Returns the catalog number of this release event.
+
+		@return: A string containing the catalog number, or None
+		"""
+		return self._catalogNumber
+	
+	def setCatalogNumber(self, catalogNumber):
+		"""Sets the catalog number of this release event.
+		
+		@param catalogNumber: A string containing the catalog number
+		"""
+		self._catalogNumber = catalogNumber
+	
+	catalogNumber = property(getCatalogNumber, setCatalogNumber,
+		doc='The catalog number of the release event')
+		
+	def getBarcode(self):
+		"""Returns the barcode of this release event.
+
+		@return: A string containing the barcode, or None
+		"""
+		return self._barcode
+	
+	def setBarcode(self, barcode):
+		"""Sets the barcode of this release event.
+		
+		@param barcode: A string containing the barcode
+		"""
+		self._barcode = barcode
+	
+	barcode = property(getBarcode, setBarcode,
+		doc='The barcode of the release event')
+		
+	def getLabel(self):
+		"""Returns a L{Label} object for the label associated with this release.
+		
+		@return: a L{Label} object, or None
+		"""
+		return self._label
+
+	def setLabel(self, label):
+		"""Sets the label of this release event.
+		
+		@param label: A L{Label} object
+		"""
+		self._label = label
+
+	label = property(getLabel, setLabel, doc='The label of the release')
+
 	def getDate(self):
 		"""Returns the date a release took place.
 
@@ -1335,6 +1727,22 @@ class ReleaseEvent(object):
 
 	date = property(getDate, setDate, doc='The date a release took place.')
 
+	def getFormat(self):
+		"""Returns the format of the release medium.
+
+		@return: a string containing a URI, or None
+		"""
+		return self._format
+
+	def setFormat(self, format):
+		"""Sets the format of the release medium.
+
+		@param format: a string containing a URI
+		"""
+		self._format = format
+
+	format = property(getFormat, setFormat,
+		doc='The format of the release medium.')
 
 
 class Disc(object):
@@ -1465,17 +1873,8 @@ class Disc(object):
 		self._tracks.append(track)
 
 
-class ArtistAlias(object):
-	"""Represents an artist alias.
-
-	An alias (the I{alias value}) is a different representation of an
-	artist's name. This may be a common misspelling or a transliteration
-	(the I{alias type}).
-
-	The I{alias script} is interesting mostly for transliterations and
-	indicates which script is used for the alias value. To represent the
-	script, ISO-15924 script codes like 'Latn', 'Cyrl', or 'Hebr' are used.
-	"""
+class AbstractAlias(object):
+	"""An abstract super class for all alias classes."""
 	def __init__(self, value=None, type_=None, script=None):
 		"""Constructor.
 
@@ -1536,6 +1935,33 @@ class ArtistAlias(object):
 	script = property(getScript, setScript, doc='The alias script.')
 
 
+class ArtistAlias(AbstractAlias):
+	"""Represents an artist alias.
+
+	An alias (the I{alias value}) is a different representation of an
+	artist's name. This may be a common misspelling or a transliteration
+	(the I{alias type}).
+
+	The I{alias script} is interesting mostly for transliterations and
+	indicates which script is used for the alias value. To represent the
+	script, ISO-15924 script codes like 'Latn', 'Cyrl', or 'Hebr' are used.
+	"""
+	pass
+
+
+class LabelAlias(AbstractAlias):
+	"""Represents a label alias.
+
+	An alias (the I{alias value}) is a different representation of a
+	label's name. This may be a common misspelling or a transliteration
+	(the I{alias type}).
+
+	The I{alias script} is interesting mostly for transliterations and
+	indicates which script is used for the alias value. To represent the
+	script, ISO-15924 script codes like 'Latn', 'Cyrl', or 'Hebr' are used.
+	"""
+	pass
+
 
 class User(object):
 	"""Represents a MusicBrainz user."""
