@@ -4,17 +4,23 @@ This module contains helper functions to make common tasks easier.
 
 @author: Matthias Friedrich <matt@mafr.de>
 """
-__revision__ = '$Id: utils.py 7386 2006-04-30 11:12:55Z matt $'
+__revision__ = '$Id: utils.py 9714 2008-03-01 09:05:48Z matt $'
 
 import re
 import urlparse
 import os.path
 
 __all__ = [
-	'extractUuid', 'extractFragment', 'getReleaseTypeName',
-	'getCountryName', 'getLanguageName', 'getScriptName',
+	'extractUuid', 'extractFragment', 'extractEntityType',
+	'getReleaseTypeName', 'getCountryName', 'getLanguageName',
+	'getScriptName',
 ]
 
+
+# A pattern to split the path part of an absolute MB URI.
+PATH_PATTERN = '^/(artist|release|track|label)/([^/]*)$'
+
+
 def extractUuid(uriStr, resType=None):
 	"""Extract the UUID part from a MusicBrainz identifier.
 
@@ -55,7 +61,7 @@ def extractUuid(uriStr, resType=None):
 	if scheme != 'http' or netloc != 'musicbrainz.org':
 		raise ValueError('%s is no MB ID.' % uriStr)
 
-	m = re.match('^/(artist|release|track)/([^/]*)$', path)
+	m = re.match(PATH_PATTERN, path)
 
 	if m:
 		if resType is None:
@@ -98,6 +104,33 @@ def extractFragment(uriStr, uriPrefix=None):
 		raise ValueError("prefix doesn't match URI %s" % uriStr)
 
 
+def extractEntityType(uriStr):
+	"""Returns the entity type an entity URI is referring to.
+
+	@param uriStr: a string containing an absolute entity URI
+
+	@return: a string containing 'artist', 'release', 'track', or 'label'
+
+	@raise ValueError: if the given URI is no valid MusicBrainz ID
+	"""
+	if uriStr is None:
+		raise ValueError('None is no valid entity URI')
+
+	(scheme, netloc, path) = urlparse.urlparse(uriStr)[:3]
+
+	if scheme == '':
+		raise ValueError('%s is no absolute MB ID.' % uriStr)
+
+	if scheme != 'http' or netloc != 'musicbrainz.org':
+		raise ValueError('%s is no MB ID.' % uriStr)
+
+	m = re.match(PATH_PATTERN, path)
+
+	if m:
+		return m.group(1)
+	else:
+		raise ValueError('%s is no valid MB ID.' % uriStr)
+
 
 def getReleaseTypeName(releaseType):
 	"""Returns the name of a release type URI.
