@@ -12,7 +12,7 @@ model <musicbrainz2.model>}.
 
 @author: Matthias Friedrich <matt@mafr.de>
 """
-__revision__ = '$Id: webservice.py 217 2006-04-06 17:34:56Z matt $'
+__revision__ = '$Id: webservice.py 9715 2008-03-01 09:21:28Z matt $'
 
 import re
 import urllib
@@ -30,7 +30,9 @@ __all__ = [
 	'WebServiceError', 'AuthenticationError', 'ConnectionError',
 	'RequestError', 'ResourceNotFoundError', 'ResponseError', 
 	'IIncludes', 'ArtistIncludes', 'ReleaseIncludes', 'TrackIncludes',
-	'IFilter', 'ArtistFilter', 'ReleaseFilter', 'TrackFilter', 'UserFilter',
+	'LabelIncludes',
+	'IFilter', 'ArtistFilter', 'ReleaseFilter', 'TrackFilter',
+	'UserFilter', 'LabelFilter',
 	'IWebService', 'WebService', 'Query',
 ]
 
@@ -77,6 +79,8 @@ class IWebService(object):
 		@param data: A string containing the data to post
 		@param version: a string containing the web service version to use
 
+		@return: a file-like object
+
 		@raise WebServiceError: in case of errors
 		"""
 		raise NotImplementedError()
@@ -226,6 +230,13 @@ class WebService(IWebService):
 		return url
 
 
+	def _openUrl(self, url, data=None):
+		userAgent = 'python-musicbrainz/' + musicbrainz2.__version__
+		req = urllib2.Request(url)
+		req.add_header('User-Agent', userAgent)
+		return self._opener.open(req, data)
+
+
 	def get(self, entity, id_, include=( ), filter={ }, version='1'):
 		"""Query the web service via HTTP-GET.
 
@@ -246,7 +257,7 @@ class WebService(IWebService):
 		self._log.debug('GET ' + url)
 
 		try:
-			return self._opener.open(url)
+			return self._openUrl(url)
 		except urllib2.HTTPError, e:
 			self._log.debug("GET failed: " + str(e))
 			if e.code == 400:   # in python 2.4: httplib.BAD_REQUEST
@@ -277,15 +288,11 @@ class WebService(IWebService):
 		"""
 		url = self._makeUrl(entity, id_, version=version, type_=None)
 
-		userAgent = 'python-musicbrainz/' + musicbrainz2.__version__
+		self._log.debug('POST ' + url)
+		self._log.debug('POST-BODY: ' + data)
 
 		try:
-			req = urllib2.Request(url)
-			req.add_header('User-Agent', userAgent)
-
-			self._log.debug('POST ' + url)
-			self._log.debug('POST-BODY: ' + data)
-			self._opener.open(req, data)
+			return self._openUrl(url, data)
 		except urllib2.HTTPError, e:
 			self._log.debug("POST failed: " + str(e))
 			if e.code == 400:   # in python 2.4: httplib.BAD_REQUEST
@@ -324,7 +331,15 @@ class WebService(IWebService):
 class IFilter(object):
 	"""A filter for collections.
 
-	This is the interface all filters have to implement.
+	This is the interface all filters have to implement. Filter classes
+	are initialized with a set of criteria and are then applied to
+	collections of items. The criteria are usually strings or integer
+	values, depending on the filter.
+
+	Note that all strings passed to filters should be unicode strings
+	(python type C{unicode}). Standard strings are converted to unicode
+	internally, but have a limitation: Only 7 Bit pure ASCII characters
+	may be used, otherwise a C{UnicodeDecodeError} is raised.
 	"""
 	def createParameters(self):
 		"""Create a list of query parameters.
@@ -343,26 +358,67 @@ class IFilter(object):
 class ArtistFilter(IFilter):
 	"""A filter for the artist collection."""
 
-	def __init__(self, name=None, limit=None):
+	def __init__(self, name=None, limit=None, offset=None, query=None):
 		"""Constructor.
 
-		@param name: a string containing the artist's name
+		The C{query} parameter may contain a query in U{Lucene syntax
+		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
+		Note that the C{name} and C{query} may not be used together.
+
+		@param name: a unicode string containing the artist's name
 		@param limit: the maximum number of artists to return
+		@param offset: start results at this zero-based offset
+		@param query: a string containing a query in Lucene syntax
 		"""
 		self._params = [
 			('name', name),
 			('limit', limit),
+			('offset', offset),
+			('query', query),
 		]
 
+		if not _paramsValid(self._params):
+			raise ValueError('invalid combination of parameters')
+
 	def createParameters(self):
-		return filter(lambda x: x[1] is not None, self._params)
+		return _createParameters(self._params)
+
+
+class LabelFilter(IFilter):
+	"""A filter for the label collection."""
+
+	def __init__(self, name=None, limit=None, offset=None, query=None):
+		"""Constructor.
+
+		The C{query} parameter may contain a query in U{Lucene syntax
+		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
+		Note that the C{name} and C{query} may not be used together.
+
+		@param name: a unicode string containing the label's name
+		@param limit: the maximum number of labels to return
+		@param offset: start results at this zero-based offset
+		@param query: a string containing a query in Lucene syntax
+		"""
+		self._params = [
+			('name', name),
+			('limit', limit),
+			('offset', offset),
+			('query', query),
+		]
+
+		if not _paramsValid(self._params):
+			raise ValueError('invalid combination of parameters')
+
+	def createParameters(self):
+		return _createParameters(self._params)
 
 
 class ReleaseFilter(IFilter):
 	"""A filter for the release collection."""
 
 	def __init__(self, title=None, discId=None, releaseTypes=None,
-			artistName=None, artistId=None, limit=None):
+			artistName=None, artistId=None, limit=None,
+			offset=None, query=None):
 		"""Constructor.
 
 		If C{discId} or C{artistId} are set, only releases matching
@@ -380,12 +436,19 @@ class ReleaseFilter(IFilter):
 		If both the C{artistName} and the C{artistId} parameter are
 		given, the server will ignore C{artistName}.
 
-		@param title: a string containing the release's title
-		@param discId: a string containing the DiscID
+		The C{query} parameter may contain a query in U{Lucene syntax
+		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
+		Note that C{query} may not be used together with the other
+		parameters except for C{limit} and C{offset}.
+
+		@param title: a unicode string containing the release's title
+		@param discId: a unicode string containing the DiscID
 		@param releaseTypes: a sequence of release type URIs
-		@param artistName: a string containing the artist's name
-		@param artistId: a string containing the artist's ID
+		@param artistName: a unicode string containing the artist's name
+		@param artistId: a unicode string containing the artist's ID
 		@param limit: the maximum number of releases to return
+		@param offset: start results at this zero-based offset
+		@param query: a string containing a query in Lucene syntax
 
 		@see: the constants in L{musicbrainz2.model.Release}
 		"""
@@ -402,10 +465,15 @@ class ReleaseFilter(IFilter):
 			('artist', artistName),
 			('artistid', artistId),
 			('limit', limit),
+			('offset', offset),
+			('query', query),
 		]
 
+		if not _paramsValid(self._params):
+			raise ValueError('invalid combination of parameters')
+
 	def createParameters(self):
-		return filter(lambda x: x[1] is not None, self._params)
+		return _createParameters(self._params)
 
 
 class TrackFilter(IFilter):
@@ -413,7 +481,8 @@ class TrackFilter(IFilter):
 
 	def __init__(self, title=None, artistName=None, artistId=None,
 			releaseTitle=None, releaseId=None,
-			duration=None, puid=None, limit=None):
+			duration=None, puid=None, limit=None, offset=None,
+			query=None):
 		"""Constructor.
 
 		If C{artistId}, C{releaseId} or C{puid} are set, only tracks
@@ -422,14 +491,21 @@ class TrackFilter(IFilter):
 		The server will ignore C{artistName} and C{releaseTitle} if
 		C{artistId} or ${releaseId} are set respectively.
 
-		@param title: a string containing the track's title
-		@param artistName: a string containing the artist's name
+		The C{query} parameter may contain a query in U{Lucene syntax
+		<http://lucene.apache.org/java/docs/queryparsersyntax.html>}.
+		Note that C{query} may not be used together with the other
+		parameters except for C{limit} and C{offset}.
+
+		@param title: a unicode string containing the track's title
+		@param artistName: a unicode string containing the artist's name
 		@param artistId: a string containing the artist's ID
-		@param releaseTitle: a string containing the release's title
+		@param releaseTitle: a unicode string containing the release's title
 		@param releaseId: a string containing the release's title
 		@param duration: the track's length in milliseconds
 		@param puid: a string containing a PUID
 		@param limit: the maximum number of releases to return
+		@param offset: start results at this zero-based offset
+		@param query: a string containing a query in Lucene syntax
 		"""
 		self._params = [
 			('title', title),
@@ -440,10 +516,15 @@ class TrackFilter(IFilter):
 			('duration', duration),
 			('puid', puid),
 			('limit', limit),
+			('offset', offset),
+			('query', query),
 		]
 
+		if not _paramsValid(self._params):
+			raise ValueError('invalid combination of parameters')
+
 	def createParameters(self):
-		return filter(lambda x: x[1] is not None, self._params)
+		return _createParameters(self._params)
 
 
 class UserFilter(IFilter):
@@ -452,13 +533,13 @@ class UserFilter(IFilter):
 	def __init__(self, name=None):
 		"""Constructor.
 
-		@param name: a string containing a MusicBrainz user name
+		@param name: a unicode string containing a MusicBrainz user name
 		"""
 		self._name = name
 
 	def createParameters(self):
 		if self._name is not None:
-			return [ ('name', self._name) ]
+			return [ ('name', self._name.encode('utf-8')) ]
 		else:
 			return [ ]
 
@@ -490,7 +571,7 @@ class ArtistIncludes(IIncludes):
 	"""
 	def __init__(self, aliases=False, releases=(), vaReleases=(),
 			artistRelations=False, releaseRelations=False,
-			trackRelations=False, urlRelations=False):
+			trackRelations=False, urlRelations=False, tags=False):
 
 		assert not isinstance(releases, basestring)
 		assert not isinstance(vaReleases, basestring)
@@ -502,6 +583,7 @@ class ArtistIncludes(IIncludes):
 			'release-rels':		releaseRelations,
 			'track-rels':		trackRelations,
 			'url-rels':		urlRelations,
+			'tags':		tags,
 		}
 
 		for elem in releases:
@@ -519,10 +601,12 @@ class ReleaseIncludes(IIncludes):
 	def __init__(self, artist=False, counts=False, releaseEvents=False,
 			discs=False, tracks=False,
 			artistRelations=False, releaseRelations=False,
-			trackRelations=False, urlRelations=False):
+			trackRelations=False, urlRelations=False,
+			labels=False, tags=False):
 		self._includes = {
 			'artist':		artist,
 			'counts':		counts,
+			'labels':		labels,
 			'release-events':	releaseEvents,
 			'discs':		discs,
 			'tracks':		tracks,
@@ -530,8 +614,14 @@ class ReleaseIncludes(IIncludes):
 			'release-rels':		releaseRelations,
 			'track-rels':		trackRelations,
 			'url-rels':		urlRelations,
+			'tags':		tags,
 		}
 
+		# Requesting labels without releaseEvents makes no sense,
+		# so we pull in releaseEvents, if necessary.
+		if labels and not releaseEvents:
+			self._includes['release-events'] = True
+
 	def createIncludeTags(self):
 		return _createIncludes(self._includes)
 
@@ -540,7 +630,7 @@ class TrackIncludes(IIncludes):
 	"""A specification on how much data to return with a track."""
 	def __init__(self, artist=False, releases=False, puids=False,
 			artistRelations=False, releaseRelations=False,
-			trackRelations=False, urlRelations=False):
+			trackRelations=False, urlRelations=False, tags=False):
 		self._includes = {
 			'artist':		artist,
 			'releases':		releases,
@@ -549,6 +639,19 @@ class TrackIncludes(IIncludes):
 			'release-rels':		releaseRelations,
 			'track-rels':		trackRelations,
 			'url-rels':		urlRelations,
+			'tags':		tags,
+		}
+
+	def createIncludeTags(self):
+		return _createIncludes(self._includes)
+
+
+class LabelIncludes(IIncludes):
+	"""A specification on how much data to return with a label."""
+	def __init__(self, aliases=False, tags=False):
+		self._includes = {
+			'aliases':		aliases,
+			'tags':		tags,
 		}
 
 	def createIncludeTags(self):
@@ -617,7 +720,7 @@ class Query(object):
 	>>> import musicbrainz2.webservice as ws
 	>>> q = ws.Query()
 	>>> artist = q.getArtistById('c0b2500e-0cef-4130-869d-732b23ed9df5')
-	>>> artist.getName()
+	>>> artist.name
 	u'Tori Amos'
 	>>> artist.sortName
 	u'Amos, Tori'
@@ -638,12 +741,12 @@ class Query(object):
 	>>> releaseId = '33dbcf02-25b9-4a35-bdb7-729455f33ad7'
 	>>> include = ws.ReleaseIncludes(artist=True, tracks=True)
 	>>> release = q.getReleaseById(releaseId, include)
-	>>> print release.title
-	Tales of a Librarian
-	>>> print release.artist.name
-	Tori Amos
-	>>> print release.tracks[0].title
-	Precious Things
+	>>> release.title
+	u'Tales of a Librarian'
+	>>> release.artist.name
+	u'Tori Amos'
+	>>> release.tracks[0].title
+	u'Precious Things'
 	>>>
 
 	Note that the query gets more expensive for the server the more
@@ -668,10 +771,10 @@ class Query(object):
 	>>> q = ws.Query()
 	>>> filter = ws.ReleaseFilter(discId='8jJklE258v6GofIqDIrE.c5ejBE-')
 	>>> results = q.getReleases(filter=filter)
-	>>> print results[0].score
+	>>> results[0].score
 	100
-	>>> print results[0].release.title
-	Under the Pink
+	>>> results[0].release.title
+	u'Under the Pink'
 	>>>
 
 	The query returns a list of results (L{wsxml.ReleaseResult} objects
@@ -683,7 +786,9 @@ class Query(object):
 
 	All filters support the C{limit} argument to limit the number of
 	results returned. This defaults to 25, but the server won't send
-	more than 100 results to save bandwidth and processing power.
+	more than 100 results to save bandwidth and processing power. Using
+	C{limit} and the C{offset} parameter, you can page through the
+	results.
 
 
 	Error Handling
@@ -728,7 +833,7 @@ class Query(object):
 
 		@param ws: a subclass instance of L{IWebService}, or None
 		@param wsFactory: a callable object which creates an object
-		@param clientId: a string containing the application's ID
+		@param clientId: a unicode string containing the application's ID
 		"""
 		if ws is None:
 			self._ws = wsFactory()
@@ -779,6 +884,30 @@ class Query(object):
 		result = self._getFromWebService('artist', '', filter=filter)
 		return result.getArtistResults()
 
+	def getLabelById(self, id_, include=None):
+		"""Returns a L{model.Label}
+		
+		If no label with that ID can be found, or there is a server problem,
+		an exception is raised.
+		
+		@param id_: a string containing the label's ID.
+
+		@raise ConnectionError: couldn't connect to server
+		@raise RequestError: invalid ID or include tags
+		@raise ResourceNotFoundError: release doesn't exist
+		@raise ResponseError: server returned invalid data
+		"""
+		uuid = mbutils.extractUuid(id_, 'label')
+		result = self._getFromWebService('label', uuid, include)
+		label = result.getLabel()
+		if label is not None:
+			return label
+		else:
+			raise ResponseError("server didn't return a label")
+	
+	def getLabels(self, filter):
+		result = self._getFromWebService('label', '', filter=filter)
+		return result.getLabelResults()
 
 	def getReleaseById(self, id_, include=None):
 		"""Returns a release.
@@ -872,7 +1001,7 @@ class Query(object):
 		See the example in L{Query} on how to supply user name and
 		password.
 
-		@param name: a string containing the user's name
+		@param name: a unicode string containing the user's name
 
 		@return: a L{User <musicbrainz2.model.User>} object
 
@@ -930,7 +1059,7 @@ class Query(object):
 		"""
 		assert self._clientId is not None, 'Please supply a client ID'
 		params = [ ]
-		params.append( ('client', self._clientId) )
+		params.append( ('client', self._clientId.encode('utf-8')) )
 
 		for (trackId, puid) in tracks2puids.iteritems():
 			trackId = mbutils.extractUuid(trackId, 'track')
@@ -941,10 +1070,97 @@ class Query(object):
 		self._ws.post('track', '', encodedStr)
 
 
+	def submitUserTags(self, entityUri, tags):
+		"""Submit folksonomy tags for an entity.
+
+		Note that all previously existing tags from the authenticated
+		user are replaced with the ones given to this method. Other
+		users' tags are not affected.
+		
+		@param entityUri: a string containing an absolute MB ID
+		@param tags: A list of either L{Tag <musicbrainz2.model.Tag>} objects
+		             or strings
+
+		@raise ValueError: invalid entityUri
+		@raise ConnectionError: couldn't connect to server
+		@raise RequestError: invalid ID, entity or tags
+		@raise AuthenticationError: invalid user name and/or password
+		"""
+		entity = mbutils.extractEntityType(entityUri)
+		uuid = mbutils.extractUuid(entityUri, entity)
+		params = (
+			('type', 'xml'),
+			('entity', entity),
+			('id', uuid),
+			('tags', ','.join([unicode(tag).encode('utf-8') for tag in tags]))
+		)
+
+		encodedStr = urllib.urlencode(params)
+
+		self._ws.post('tag', '', encodedStr)
+
+
+	def getUserTags(self, entityUri):
+		"""Returns a list of folksonomy tags a user has applied to an entity.
+
+		The given parameter has to be a fully qualified MusicBrainz ID, as
+		returned by other library functions.
+		
+		Note that this method only works if a valid user name and
+		password have been set. Only the tags the authenticated user
+		applied to the entity will be returned. If username and/or
+		password are incorrect, an AuthenticationError is raised.
+		
+		This method will return a list of L{Tag <musicbrainz2.model.Tag>}
+		objects.
+		
+		@param entityUri: a string containing an absolute MB ID
+		
+		@raise ValueError: invalid entityUri
+  		@raise ConnectionError: couldn't connect to server
+		@raise RequestError: invalid ID or entity
+		@raise AuthenticationError: invalid user name and/or password
+		"""
+		entity = mbutils.extractEntityType(entityUri)
+		uuid = mbutils.extractUuid(entityUri, entity)
+		params = { 'entity': entity, 'id': uuid }
+		
+		stream = self._ws.get('tag', '', filter=params)
+		try:
+			parser = MbXmlParser()
+			result = parser.parse(stream)
+		except ParseError, e:
+			raise ResponseError(str(e), e)
+		
+		return result.getTagList()
+		
+
 def _createIncludes(tagMap):
 	selected = filter(lambda x: x[1] == True, tagMap.items())
 	return map(lambda x: x[0], selected)
 
+def _createParameters(params):
+	"""Remove (x, None) tuples and encode (x, str/unicode) to utf-8."""
+	ret = [ ]
+	for p in params:
+		if isinstance(p[1], (str, unicode)):
+			ret.append( (p[0], p[1].encode('utf-8')) )
+		elif p[1] is not None:
+			ret.append(p)
+
+	return ret
+
+def _paramsValid(params):
+	"""Check if the query parameter collides with other parameters."""
+	tmp = [ ]
+	for name, value in params:
+		if value is not None and name not in ('offset', 'limit'):
+			tmp.append(name)
+
+	if 'query' in tmp and len(tmp) > 1:
+		return False
+	else:
+		return True
 
 if __name__ == '__main__':
 	import doctest
