@@ -7,7 +7,7 @@ MusicBrainz webservice.
 There are also DOM helper functions in this module used by the parser which
 probably aren't useful to users.
 """
-__revision__ = '$Id: wsxml.py 8456 2006-09-03 08:52:14Z matt $'
+__revision__ = '$Id: wsxml.py 9724 2008-03-09 08:34:11Z matt $'
 
 import re
 import logging
@@ -24,7 +24,8 @@ from musicbrainz2.model import NS_MMD_1, NS_REL_1, NS_EXT_1
 __all__ = [
 	'DefaultFactory', 'Metadata', 'ParseError',
 	'MbXmlParser', 'MbXmlWriter',
-	'ArtistResult', 'ReleaseResult', 'TrackResult',
+	'AbstractResult',
+	'ArtistResult', 'ReleaseResult', 'TrackResult', 'LabelResult'
 ]
 
 
@@ -41,6 +42,9 @@ class DefaultFactory(object):
 	def newDisc(self): return model.Disc()
 	def newArtistAlias(self): return model.ArtistAlias()
 	def newUser(self): return model.User()
+	def newLabel(self): return model.Label()
+	def newLabelAlias(self): return model.LabelAlias()
+	def newTag(self): return model.Tag()
 
 
 class ParseError(Exception):
@@ -74,9 +78,20 @@ class Metadata(object):
 		self._artist = None
 		self._release = None
 		self._track = None
+		self._label = None
 		self._artistResults = [ ]
+		self._artistResultsOffset = None
+		self._artistResultsCount = None
 		self._releaseResults = [ ]
+		self._releaseResultsOffset = None
+		self._releaseResultsCount = None
 		self._trackResults = [ ]
+		self._trackResultsOffset = None
+		self._trackResultsCount = None
+		self._labelResults = [ ]
+		self._labelResultsOffset = None
+		self._labelResultsCount = None
+		self._tagList = [ ]
 		self._userList = [ ]
 
 	def getArtist(self):
@@ -86,6 +101,14 @@ class Metadata(object):
 		self._artist = artist
 
 	artist = property(getArtist, setArtist, doc='An Artist object.')
+	
+	def getLabel(self):
+		return self._label
+	
+	def setLabel(self, label):
+		self._label = label
+	
+	label = property(getLabel, setLabel, doc='A Label object.')
 
 	def getRelease(self):
 		return self._release
@@ -113,6 +136,118 @@ class Metadata(object):
 	artistResults = property(getArtistResults,
 		doc='A list of ArtistResult objects.')
 
+	def getArtistResultsOffset(self):
+		"""Returns the offset of the artist result list.
+
+		The offset is used for paging through the result list. It
+		is zero-based.
+
+		@return: an integer containing the offset, or None
+
+		@see: L{getArtistResults}, L{getArtistResultsCount}
+		"""
+		return self._artistResultsOffset
+
+	def setArtistResultsOffset(self, value):
+		"""Sets the offset of the artist result list.
+
+		@param value: an integer containing the offset, or None
+
+		@see: L{getArtistResultsOffset}
+		"""
+		self._artistResultsOffset = value
+
+	artistResultsOffset = property(
+		getArtistResultsOffset, setArtistResultsOffset,
+		doc='The offset of the artist results.')
+
+	def getArtistResultsCount(self):
+		"""Returns the total number of results available.
+	
+		This may or may not match with the number of elements that
+		L{getArtistResults} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setArtistResultsCount}, L{getArtistResultsOffset}
+		"""
+		return self._artistResultsCount
+
+	def setArtistResultsCount(self, value):
+		"""Sets the total number of available results.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getArtistResults}, L{setArtistResultsOffset}
+		"""
+		self._artistResultsCount = value
+
+	artistResultsCount = property(
+		getArtistResultsCount, setArtistResultsCount,
+		doc='The total number of artists results.')
+
+	def getLabelResults(self):
+		"""Returns a label result list.
+		
+		@return: a list of L{LabelResult} objects.
+		"""
+		return self._labelResults
+	
+	labelResults = property(getLabelResults,
+		doc='A list of LabelResult objects')
+
+	def getLabelResultsOffset(self):
+		"""Returns the offset of the label result list.
+
+		The offset is used for paging through the result list. It
+		is zero-based.
+
+		@return: an integer containing the offset, or None
+
+		@see: L{getLabelResults}, L{getLabelResultsCount}
+		"""
+		return self._labelResultsOffset
+
+	def setLabelResultsOffset(self, value):
+		"""Sets the offset of the label result list.
+
+		@param value: an integer containing the offset, or None
+
+		@see: L{getLabelResultsOffset}
+		"""
+		self._labelResultsOffset = value
+
+	labelResultsOffset = property(
+		getLabelResultsOffset, setLabelResultsOffset,
+		doc='The offset of the label results.')
+
+	def getLabelResultsCount(self):
+		"""Returns the total number of results available.
+	
+		This may or may not match with the number of elements that
+		L{getLabelResults} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setLabelResultsCount}, L{getLabelResultsOffset}
+		"""
+		return self._labelResultsCount
+
+	def setLabelResultsCount(self, value):
+		"""Sets the total number of available results.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getLabelResults}, L{setLabelResultsOffset}
+		"""
+		self._labelResultsCount = value
+
+	labelResultsCount = property(
+		getLabelResultsCount, setLabelResultsCount,
+		doc='The total number of label results.')
+
 	def getReleaseResults(self):
 		"""Returns a release result list. 
 
@@ -123,6 +258,57 @@ class Metadata(object):
 	releaseResults = property(getReleaseResults,
 		doc='A list of ReleaseResult objects.')
 
+	def getReleaseResultsOffset(self):
+		"""Returns the offset of the release result list.
+
+		The offset is used for paging through the result list. It
+		is zero-based.
+
+		@return: an integer containing the offset, or None
+
+		@see: L{getReleaseResults}, L{getReleaseResultsCount}
+		"""
+		return self._releaseResultsOffset
+
+	def setReleaseResultsOffset(self, value):
+		"""Sets the offset of the release result list.
+
+		@param value: an integer containing the offset, or None
+
+		@see: L{getReleaseResultsOffset}
+		"""
+		self._releaseResultsOffset = value
+
+	releaseResultsOffset = property(
+		getReleaseResultsOffset, setReleaseResultsOffset,
+		doc='The offset of the release results.')
+
+	def getReleaseResultsCount(self):
+		"""Returns the total number of results available.
+	
+		This may or may not match with the number of elements that
+		L{getReleaseResults} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setReleaseResultsCount}, L{getReleaseResultsOffset}
+		"""
+		return self._releaseResultsCount
+
+	def setReleaseResultsCount(self, value):
+		"""Sets the total number of available results.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getReleaseResults}, L{setReleaseResultsOffset}
+		"""
+		self._releaseResultsCount = value
+
+	releaseResultsCount = property(
+		getReleaseResultsCount, setReleaseResultsCount,
+		doc='The total number of release results.')
+
 	def getTrackResults(self):
 		"""Returns a track result list. 
 
@@ -133,6 +319,69 @@ class Metadata(object):
 	trackResults = property(getTrackResults,
 		doc='A list of TrackResult objects.')
 
+	def getTrackResultsOffset(self):
+		"""Returns the offset of the track result list.
+
+		The offset is used for paging through the result list. It
+		is zero-based.
+
+		@return: an integer containing the offset, or None
+
+		@see: L{getTrackResults}, L{getTrackResultsCount}
+		"""
+		return self._trackResultsOffset
+
+	def setTrackResultsOffset(self, value):
+		"""Sets the offset of the track result list.
+
+		@param value: an integer containing the offset, or None
+
+		@see: L{getTrackResultsOffset}
+		"""
+		self._trackResultsOffset = value
+
+	trackResultsOffset = property(
+		getTrackResultsOffset, setTrackResultsOffset,
+		doc='The offset of the track results.')
+
+	def getTrackResultsCount(self):
+		"""Returns the total number of results available.
+	
+		This may or may not match with the number of elements that
+		L{getTrackResults} returns. If the count is higher than
+		the list, it indicates that the list is incomplete.
+
+		@return: an integer containing the count, or None
+
+		@see: L{setTrackResultsCount}, L{getTrackResultsOffset}
+		"""
+		return self._trackResultsCount
+
+	def setTrackResultsCount(self, value):
+		"""Sets the total number of available results.
+
+		@param value: an integer containing the count, or None
+
+		@see: L{getTrackResults}, L{setTrackResultsOffset}
+		"""
+		self._trackResultsCount = value
+
+	trackResultsCount = property(
+		getTrackResultsCount, setTrackResultsCount,
+		doc='The total number of track results.')
+
+
+	def getTagList(self):
+		"""Returns a list of tags.
+
+		@return: a list of L{model.Tag} objects
+		"""
+		return self._tagList
+
+	tagResults = property(getTagList,
+		doc='A list of Tag objects.')
+
+
 	# MusicBrainz extension to the schema
 	def getUserList(self):
 		"""Returns a list of users.
@@ -147,7 +396,33 @@ class Metadata(object):
 		doc='A list of User objects.')
 
 
-class ArtistResult(object):
+class AbstractResult(object):
+	"""The abstract representation of a result.
+
+	A result is an instance of some kind (Artist, Release, ...)
+	associated with a score.
+	"""
+
+	def __init__(self, score):
+		self._score = score
+
+	def getScore(self):
+		"""Returns the result score.
+
+		The score indicates how good this result matches the search
+		parameters. The higher the value, the better the match.
+
+		@return: an int between 0 and 100 (both inclusive), or None
+		"""
+		return self._score
+
+	def setScore(self, score):
+		self._score = score
+
+	score = property(getScore, setScore, doc='The relevance score.')
+
+
+class ArtistResult(AbstractResult):
 	"""Represents an artist result.
 
 	An ArtistResult consists of a I{score} and an artist. The score is a
@@ -155,8 +430,8 @@ class ArtistResult(object):
 	match.
 	"""
 	def __init__(self, artist, score):
+		super(ArtistResult, self).__init__(score)
 		self._artist = artist
-		self._score = score
 
 	def getArtist(self):
 		"""Returns an Artist object.
@@ -170,23 +445,8 @@ class ArtistResult(object):
 
 	artist = property(getArtist, setArtist, doc='An Artist object.')
 
-	def getScore(self):
-		"""Returns the result score.
 
-		The score indicates how good this result matches the search
-		parameters. The higher the value, the better the match.
-
-		@return: an int between 0 and 100 (both inclusive), or None
-		"""
-		return self._score
-
-	def setScore(self, score):
-		self._score = score
-
-	score = property(getScore, setScore, doc='The relevance score.')
-
-
-class ReleaseResult(object):
+class ReleaseResult(AbstractResult):
 	"""Represents a release result.
 
 	A ReleaseResult consists of a I{score} and a release. The score is a
@@ -194,8 +454,8 @@ class ReleaseResult(object):
 	match.
 	"""
 	def __init__(self, release, score):
+		super(ReleaseResult, self).__init__(score)
 		self._release = release
-		self._score = score
 
 	def getRelease(self):
 		"""Returns a Release object.
@@ -209,23 +469,8 @@ class ReleaseResult(object):
 
 	release = property(getRelease, setRelease, doc='A Release object.')
 
-	def getScore(self):
-		"""Returns the result score.
 
-		The score indicates how good this result matches the search
-		parameters. The higher the value, the better the match.
-
-		@return: an int between 0 and 100 (both inclusive), or None
-		"""
-		return self._score
-
-	def setScore(self, score):
-		self._score = score
-
-	score = property(getScore, setScore, doc='The relevance score.')
-
-
-class TrackResult(object):
+class TrackResult(AbstractResult):
 	"""Represents a track result.
 
 	A TrackResult consists of a I{score} and a track. The score is a
@@ -233,8 +478,8 @@ class TrackResult(object):
 	match.
 	"""
 	def __init__(self, track, score):
+		super(TrackResult, self).__init__(score)
 		self._track = track
-		self._score = score
 
 	def getTrack(self):
 		"""Returns a Track object.
@@ -248,20 +493,29 @@ class TrackResult(object):
 
 	track = property(getTrack, setTrack, doc='A Track object.')
 
-	def getScore(self):
-		"""Returns the result score.
 
-		The score indicates how good this result matches the search
-		parameters. The higher the value, the better the match.
+class LabelResult(AbstractResult):
+	"""Represents a label result.
 
-		@return: an int between 0 and 100 (both inclusive), or None
+	An LabelResult consists of a I{score} and a label. The score is a
+	number between 0 and 100, where a higher number indicates a better
+	match.
+	"""
+	def __init__(self, label, score):
+		super(LabelResult, self).__init__(score)
+		self._label = label
+
+	def getLabel(self):
+		"""Returns a Label object.
+
+		@return: a L{musicbrainz2.model.Label} object
 		"""
-		return self._score
+		return self._label
 
-	def setScore(self, score):
-		self._score = score
+	def setLabel(self, label):
+		self._label = label
 
-	score = property(getScore, setScore, doc='The relevance score.')
+	label = property(getLabel, setLabel, doc='A Label object.')
 
 
 class MbXmlParser(object):
@@ -344,17 +598,35 @@ class MbXmlParser(object):
 
 		for node in _getChildElements(metadata):
 			if _matches(node, 'artist'):
-				md.setArtist(self._createArtist(node))
+				md.artist = self._createArtist(node)
 			elif _matches(node, 'release'):
-				md.setRelease(self._createRelease(node))
+				md.release = self._createRelease(node)
 			elif _matches(node, 'track'):
-				md.setTrack(self._createTrack(node))
+				md.track = self._createTrack(node)
+			elif _matches(node, 'label'):
+				md.label = self._createLabel(node)
 			elif _matches(node, 'artist-list'):
+				(offset, count) = self._getListAttrs(node)
+				md.artistResultsOffset = offset
+				md.artistResultsCount = count
 				self._addArtistResults(node, md.getArtistResults())
 			elif _matches(node, 'release-list'):
+				(offset, count) = self._getListAttrs(node)
+				md.releaseResultsOffset = offset
+				md.releaseResultsCount = count
 				self._addReleaseResults(node, md.getReleaseResults())
 			elif _matches(node, 'track-list'):
+				(offset, count) = self._getListAttrs(node)
+				md.trackResultsOffset = offset
+				md.trackResultsCount = count
 				self._addTrackResults(node, md.getTrackResults())
+			elif _matches(node, 'label-list'):
+				(offset, count) = self._getListAttrs(node)
+				md.labelResultsOffset = offset
+				md.labelResultsCount = count
+				self._addLabelResults(node, md.getLabelResults())
+			elif _matches(node, 'tag-list'):
+				self._addTagsToList(node, md.getTagList())
 			elif _matches(node, 'user-list', NS_EXT_1):
 				self._addUsersToList(node, md.getUserList())
 
@@ -382,6 +654,12 @@ class MbXmlParser(object):
 			if track is not None:
 				resultList.append(TrackResult(track, score))
 
+	def _addLabelResults(self, listNode, resultList):
+		for c in _getChildElements(listNode):
+			label = self._createLabel(c)
+			score = _getIntAttr(c, 'score', 0, 100, ns=NS_EXT_1)
+			if label is not None:
+				resultList.append(LabelResult(label, score))
 
 	def _addReleasesToList(self, listNode, resultList):
 		self._addToList(listNode, resultList, self._createRelease)
@@ -392,6 +670,14 @@ class MbXmlParser(object):
 	def _addUsersToList(self, listNode, resultList):
 		self._addToList(listNode, resultList, self._createUser)
 
+	def _addTagsToList(self, listNode, resultList):
+		self._addToList(listNode, resultList, self._createTag)
+
+	def _addTagsToEntity(self, listNode, entity):
+		for node in _getChildElements(listNode):
+			tag = self._createTag(node)
+			entity.addTag(tag)
+	
 	def _addToList(self, listNode, resultList, creator):
 		for c in _getChildElements(listNode):
 			resultList.append(creator(c))
@@ -425,9 +711,37 @@ class MbXmlParser(object):
 				self._addReleasesToList(node, artist.getReleases())
 			elif _matches(node, 'relation-list'):
 				self._addRelationsToEntity(node, artist)
+			elif _matches(node, 'tag-list'):
+				self._addTagsToEntity(node, artist)
 
 		return artist
 
+	def _createLabel(self, labelNode):
+		label = self._factory.newLabel()
+		label.setId(_getIdAttr(labelNode, 'id', 'label'))
+		label.setType(_getUriAttr(labelNode, 'type'))
+		
+		for node in _getChildElements(labelNode):
+			if _matches(node, 'name'):
+				label.setName(_getText(node))
+			if _matches(node, 'sort-name'):
+				label.setSortName(_getText(node))
+			elif _matches(node, 'disambiguation'):
+				label.setDisambiguation(_getText(node))
+			elif _matches(node, 'label-code'):
+				label.setCode(_getText(node))
+			elif _matches(node, 'country'):
+				country = _getText(node, '^[A-Z]{2}$')
+				label.setCountry(country)
+			elif _matches(node, 'life-span'):
+				label.setBeginDate(_getDateAttr(node, 'begin'))
+				label.setEndDate(_getDateAttr(node, 'end'))
+			elif _matches(node, 'alias-list'):
+				self._addLabelAliases(node, label)
+			elif _matches(node, 'tag-list'):
+				self._addTagsToEntity(node, label)
+
+		return label
 
 	def _createRelease(self, releaseNode):
 		release = self._factory.newRelease()
@@ -458,6 +772,8 @@ class MbXmlParser(object):
 				self._addTracksToList(node, release.getTracks())
 			elif _matches(node, 'relation-list'):
 				self._addRelationsToEntity(node, release)
+			elif _matches(node, 'tag-list'):
+				self._addTagsToEntity(node, release)
 
 		return release
 
@@ -467,10 +783,24 @@ class MbXmlParser(object):
 			if _matches(node, 'event'):
 				country = _getAttr(node, 'country', '^[A-Z]{2}$')
 				date = _getDateAttr(node, 'date')
+				catalogNumber = _getAttr(node, 'catalog-number')
+				barcode = _getAttr(node, 'barcode')
+				format = _getUriAttr(node, 'format')
+
+				# The date attribute is mandatory. If it isn't present,
+				# we don't add anything from this release event.
 				if date is not None:
 					event = self._factory.newReleaseEvent()
 					event.setCountry(country)
 					event.setDate(date)
+					event.setCatalogNumber(catalogNumber)
+					event.setBarcode(barcode)
+					event.setFormat(format)
+					
+					for subNode in _getChildElements(node):
+						if _matches(subNode, 'label'):
+							event.setLabel(self._createLabel(subNode))
+					
 					release.addReleaseEvent(event)
 
 
@@ -487,13 +817,25 @@ class MbXmlParser(object):
 		for node in _getChildElements(aliasListNode):
 			if _matches(node, 'alias'):
 				alias = self._factory.newArtistAlias()
-				alias.setValue(_getText(node))
-				alias.setType(_getUriAttr(node, 'type'))
-				alias.setScript(_getAttr(node, 'script',
-					'^[A-Z][a-z]{3}$'))
+				self._initializeAlias(alias, node)
 				artist.addAlias(alias)
 
 
+	def _addLabelAliases(self, aliasListNode, label):
+		for node in _getChildElements(aliasListNode):
+			if _matches(node, 'alias'):
+				alias = self._factory.newLabelAlias()
+				self._initializeAlias(alias, node)
+				label.addAlias(alias)
+
+
+	def _initializeAlias(self, alias, node):
+		alias.setValue(_getText(node))
+		alias.setType(_getUriAttr(node, 'type'))
+		alias.setScript(_getAttr(node, 'script',
+			'^[A-Z][a-z]{3}$'))
+
+
 	def _createTrack(self, trackNode):
 		track = self._factory.newTrack()
 		track.setId(_getIdAttr(trackNode, 'id', 'track'))
@@ -511,6 +853,8 @@ class MbXmlParser(object):
 				self._addPuids(node, track)
 			elif _matches(node, 'relation-list'):
 				self._addRelationsToEntity(node, track)
+			elif _matches(node, 'tag-list'):
+				self._addTagsToEntity(node, track)
 
 		return track
 
@@ -528,7 +872,13 @@ class MbXmlParser(object):
 
 		return user
 
+	def _createTag(self, tagNode):
+		tag = self._factory.newTag()
+		tag.value = _getText(tagNode)
+		tag.count = _getIntAttr(tagNode, 'count')
+		return tag
 
+	
 	def _addPuids(self, puidListNode, track):
 		for node in _getChildElements(puidListNode):
 			if _matches(node, 'puid') and node.hasAttribute('id'):
@@ -672,29 +1022,48 @@ class MbXmlWriter(object):
 		self._writeArtist(xml, metadata.getArtist())
 		self._writeRelease(xml, metadata.getRelease())
 		self._writeTrack(xml, metadata.getTrack())
+		self._writeLabel(xml, metadata.getLabel())
 
-		# TODO: count and offset
 		if len(metadata.getArtistResults()) > 0:
-			xml.start('artist-list')
+			xml.start('artist-list', {
+				'offset': metadata.artistResultsOffset,
+				'count': metadata.artistResultsCount,
+			})
 			for result in metadata.getArtistResults():
 				self._writeArtist(xml, result.getArtist(),
 					result.getScore())
 			xml.end()
 
 		if len(metadata.getReleaseResults()) > 0:
-			xml.start('release-list')
+			xml.start('release-list', {
+				'offset': metadata.releaseResultsOffset,
+				'count': metadata.releaseResultsCount,
+			})
 			for result in metadata.getReleaseResults():
 				self._writeRelease(xml, result.getRelease(),
 					result.getScore())
 			xml.end()
 
 		if len(metadata.getTrackResults()) > 0:
-			xml.start('track-list')
+			xml.start('track-list', {
+				'offset': metadata.trackResultsOffset,
+				'count': metadata.trackResultsCount,
+			})
 			for result in metadata.getTrackResults():
 				self._writeTrack(xml, result.getTrack(),
 					result.getScore())
 			xml.end()
 
+		if len(metadata.getLabelResults()) > 0:
+			xml.start('label-list', {
+				'offset': metadata.labelResultsOffset,
+				'count': metadata.labelResultsCount,
+			})
+			for result in metadata.getLabelResults():
+				self._writeLabel(xml, result.getLabel(),
+					result.getScore())
+			xml.end()
+
 		xml.end()
 
 
@@ -723,7 +1092,7 @@ class MbXmlWriter(object):
 					'type': alias.getType(),
 					'script': alias.getScript(),
 				})
-			xml.end('alias-list')
+			xml.end()
 
 		if len(artist.getReleases()) > 0:
 			xml.start('release-list')
@@ -742,13 +1111,13 @@ class MbXmlWriter(object):
 			return
 
 		types = [mbutils.extractFragment(t) for t in release.getTypes()]
-		typeStr = None
+		typesStr = None
 		if len(types) > 0:
 			typesStr = ' '.join(types)
 
 		xml.start('release', {
 			'id': mbutils.extractUuid(release.getId()),
-			'type': typeStr,
+			'type': typesStr,
 			'ext:score': score,
 		})
 
@@ -764,10 +1133,7 @@ class MbXmlWriter(object):
 		if len(release.getReleaseEvents()) > 0:
 			xml.start('release-event-list')
 			for event in release.getReleaseEvents():
-				xml.elem('alias', None, {
-					'country': event.getCountry(),
-					'date': event.getDate()
-				})
+				self._writeReleaseEvent(xml, event)
 			xml.end()
 
 		if len(release.getDiscs()) > 0:
@@ -791,6 +1157,20 @@ class MbXmlWriter(object):
 		xml.end()
 
 
+	def _writeReleaseEvent(self, xml, event):
+		xml.start('event', {
+			'country': event.getCountry(),
+			'date': event.getDate(),
+			'catalog-number': event.getCatalogNumber(),
+			'barcode': event.getBarcode(),
+			'format': event.getFormat()
+		})
+
+		self._writeLabel(xml, event.getLabel())
+
+		xml.end()
+
+
 	def _writeTrack(self, xml, track, score=None):
 		if track is None:
 			return
@@ -823,6 +1203,41 @@ class MbXmlWriter(object):
 		xml.end()
 
 
+	def _writeLabel(self, xml, label, score=None):
+		if label is None:
+			return
+
+		xml.start('label', {
+			'id': mbutils.extractUuid(label.getId()),
+			'type': mbutils.extractFragment(label.getType()),
+			'ext:score': score,
+		})
+
+		xml.elem('name', label.getName())
+		xml.elem('sort-name', label.getSortName())
+		xml.elem('disambiguation', label.getDisambiguation())
+		xml.elem('life-span', None, {
+			'begin': label.getBeginDate(),
+			'end': label.getEndDate(),
+		})
+
+		if len(label.getAliases()) > 0:
+			xml.start('alias-list')
+			for alias in label.getAliases():
+				xml.elem('alias', alias.getValue(), {
+					'type': alias.getType(),
+					'script': alias.getScript(),
+				})
+			xml.end()
+
+		# TODO: releases, artists
+
+		self._writeRelationList(xml, label)
+		# TODO: extensions
+
+		xml.end()
+
+
 	def _writeRelationList(self, xml, entity):
 		for tt in entity.getRelationTargetTypes():
 			xml.start('relation-list', {
@@ -886,7 +1301,7 @@ def _getChildElements(parentNode):
 	return children
 
 
-def _getText(element):
+def _getText(element, regex=None, default=None):
 	"""Returns the text content of the given xml.dom.Element.
 
 	This function simply fetches all contained text nodes, so the element
@@ -897,7 +1312,10 @@ def _getText(element):
 		if node.nodeType == node.TEXT_NODE:
 			res += node.data
 
-	return res
+	if regex is None or re.match(regex, res):
+		return res
+	else:
+		return default
 
 
 def _getPositiveIntText(element):
@@ -1003,10 +1421,9 @@ def _getBooleanAttr(element, attrName):
 def _getDirectionAttr(element, attrName):
 	"""Gets the Relation reading direction from an attribute."""
 	regex = '^\s*(' + '|'.join((
-				model.Relation.DIR_BOTH,
 				model.Relation.DIR_FORWARD,
 				model.Relation.DIR_BACKWARD)) + ')\s*$'
-	return _getAttr(element, 'direction', regex, model.Relation.DIR_BOTH)
+	return _getAttr(element, 'direction', regex, model.Relation.DIR_NONE)
 
 
 def _makeAbsoluteUri(prefix, uriStr):
