@@ -1,6 +1,6 @@
 """Tests for various model classes."""
 import unittest
-from musicbrainz2.model import Artist, Release, Track, Relation, NS_REL_1
+from musicbrainz2.model import Artist, Release, Track, Relation, Tag, NS_REL_1
 
 class MiscModelTest(unittest.TestCase):
 	
@@ -40,7 +40,7 @@ class MiscModelTest(unittest.TestCase):
 		self.assertEquals(len(rel5), 0)
 
 		rel6 = artist.getRelations(Relation.TO_RELEASE,
-			NS_REL_1 + 'Producer', [NS_REL_1 + 'Co'], 'both')
+			NS_REL_1 + 'Producer', [NS_REL_1 + 'Co'], 'none')
 		self.assertEquals(len(rel6), 1)
 
 		rel6 = artist.getRelations(Relation.TO_RELEASE,
@@ -72,3 +72,25 @@ class MiscModelTest(unittest.TestCase):
 		r.getTracks()[1].setArtist(Artist(id_=1))
 		self.assert_( r.isSingleArtistRelease() )
 
+	def testTags(self):
+		a = Artist()
+		a.addTag(Tag('foo', 1))
+		a.addTag(Tag('bar', 2))
+		a.addTag(Tag('bar', 5))
+
+		self.assertEquals(len(a.tags), 2)
+		self.assertEquals(a.getTag('foo').count, 1)
+		self.assertEquals(a.getTag('bar').count, 7)
+
+
+class TagTest(unittest.TestCase):
+	
+	def test__str__(self):
+		self.assertEquals("foo", str(Tag(u"foo")))
+		self.assertRaises(UnicodeEncodeError, str, Tag(u"f\u014do"))
+	
+	def test__unicode__(self):
+		self.assertEquals(u"foo", unicode(Tag(u"foo")))
+		self.assertEquals(u"f\u014do", unicode(Tag(u"f\u014do")))
+
+# EOF
